# Copyright 2005-2012 Canonical Ltd.  All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from operator import attrgetter

from oopstools.oops.models import (
    AppInstance,
    Classification,
    Infestation,
    Oops,
    Prefix,
    Report,
)
from oopstools.oops.test.testcase import TestCase


class ReportTests(TestCase):

    def test_no_report(self):
        resp = self.client.get('/reports/some-report/')
        self.assertEqual(404, resp.status_code)

    def test_inactive_report(self):
        report_name = 'areport'
        Report.objects.create(name=report_name, active=False)
        resp = self.client.get('/reports/%s/' % (report_name,))
        self.assertEqual(404, resp.status_code)

    def test_no_recent_oopses(self):
        report_name = 'areport'
        Report.objects.create(name=report_name, active=True)
        resp = self.client.get('/reports/%s/' % (report_name,))
        self.assertEqual(200, resp.status_code)
        self.assertQuerysetEqual(resp.context['recent'].object_list, [])

    def make_oops(self, prefix):
        classification = Classification.objects.create(
                title=self.getUniqueString(prefix="classification"))
        infestation = Infestation.objects.create(
                exception_type=self.getUniqueString(prefix="exc_type"),
                exception_value=self.getUniqueString(prefix="exc_value"))
        return Oops.objects.create(
                prefix=prefix, classification=classification,
                oopsinfestation=infestation, statements_count=100,
                appinstance=prefix.appinstance, total_time=3,
                date=datetime.now())

    def make_prefix(self):
        appinstance = AppInstance.objects.create(
                title=self.getUniqueString(prefix="appinstance"))
        return Prefix.objects.create(
                value=self.getUniqueString(prefix="prefix"),
                appinstance=appinstance)

    def test_recent_oopses(self):
        report_name = 'areport'
        report = Report.objects.create(name=report_name, active=True)
        prefix = self.make_prefix()
        report.prefixes.add(prefix)
        oops = self.make_oops(prefix)
        resp = self.client.get('/reports/%s/' % (report_name,))
        self.assertEqual(200, resp.status_code)
        self.assertQuerysetEqual(
            resp.context['recent'].object_list, [oops.pk], transform=attrgetter('pk'))
